#!/bin/bash

set -xeuo pipefail

#### qemu-system tests.

## Test native arch on KVM hypervisor.
# It's not guilt of QEMU if KVM cannot be tested, but if KVM could be tested
# take it seriously.
if [ -c /dev/kvm ]; then
	# Perhaps KVM could be tested.
	toilet -f bigascii12 'KVM'
	# Test using using rpm-build-vm if there's non-dummy script.
       	if [ -d /usr/lib/vm-run ]; then
		timeout 100 vm-run --kvm=cond toilet -w222 'KVM OK'
	else
		toilet -w222 'KVM test skipped'
	fi
else
	toilet 'No KVM'
fi

## Test TCG emulation.
toilet -f bigascii12 'TCG'

# Test native arch in TCG emulation using rpm-build-vm.
if [ -d /usr/lib/vm-run ]; then
	timeout 300 vm-run --tcg toilet -w222 'Native TCG OK'
fi

# Cleanup vm-run fallout.
rm -f /tmp/vm.* /tmp/initramfs-*.img

# TODO: Test other architectures using qa-qemu-vmlinuz.

check() {
	if timeout 300 "$@" |& sed "s/^/$A>/" | cat -v > boot.log; then
		grep "Linux version" boot.log
		# Booted until initrd. Good.
		grep "/dev/root: Can't open blockdev" boot.log
	else
		cat boot.log
		exit 1
	fi
	rm boot.log
}

for a in $(ls /usr/lib/qemu-vmlinuz/); do
	OPTS="-accel tcg -serial mon:stdio -nodefaults -nographic -no-reboot"
	APPEND="debug panic=-1"
	A="$a"
	case $a in
		aarch64) APPEND+=" console=ttyAMA0" OPTS+=" -M virt -cpu max" ;;
		armh)    APPEND+=" console=ttyAMA0" OPTS+=" -M virt,highmem=off -cpu max" A="arm" ;;
		x86_64)  APPEND+=" console=ttyS0" ;;
		i586)    APPEND+=" console=ttyS0" A="i386" ;;
		ppc64le) APPEND+=" console=hvc0" A="ppc64" ;;
	esac
	toilet "$a"
	check qemu-system-$A $OPTS -kernel /usr/lib/qemu-vmlinuz/$a/vmlinuz-* -append "$APPEND"
done

#### TODO: qemu-user tests.
# Need to package something for it, perhaps /bin/true from all arches?
